// By Christian Bertram, January 2019.  https://github.com/cbertam

// LICENSE: You are very welcome to share and/or modify this file, as long as you give
// credit to me, link back to this original version and don't use this work
// for commercial purposes.

// If you create a derivative version of my model (A new member of the Struder
// family? :)) feel free to tell me about it!  If you have trouble modifying
// this work, you can write a comment on myminifactory.com, and I'll try to
// help.

// For printing Phil Struder, you will need
// * 1 x nema17_cap_face();
// * 1 x magnet_mount();
// * 1 x gear_front_eyes();
// * 2 x back_clip();
//
// There are some other modules in here too.  You might find them useful. :)

back_clip();

use <MCAD/involute_gears.scad>

$fa = 1;
$fs = 1;

// Magnet adapter for if you don't have an Ø8 mm magnet
module magnet_adapter(h1=6.5, h2=2*1.7+1, d1=8.6, d2=4.2, cube_h=1.8, cube_l=5) {
  difference() {
    cylinder(h=h1, d=d1);
    translate([0, 0, -1]) cylinder(h=h2+1, d=d2);
  }
  translate([0, 0, h1+cube_h/2]) cube([cube_l, cube_l, cube_h], center=true);
}

// Magnet adapter for if you don't have an Ø8 mm magnet.  This one is better if you have Ø4 mm magnets.
module magnet_adapter_2(h1=6.5, h2=2*1.7+1, d1=8.6, d2=4.2, cube_h=1.8, cube_l=5) {
  difference() {
    cylinder(h=h1, d=d1);
    translate([ d2/2, 0, -1]) cylinder(h=h2+1, d=d2);
    translate([-d2/2, 0, -1]) cylinder(h=h2+1, d=d2);
  }
  translate([0, 0, h1+cube_h/2]) cube([cube_l, cube_l, cube_h], center=true);
}

// The mount to glue a Ø8 x 3 mm magnet to.  So you can clip it to a gear.
module magnet_mount(h1=6.5-3, d=8, cube_h=1.8, cube_l=5) {
  cylinder(h=h1, d=d);
  translate([-cube_l/2, -cube_l/2, h1]) cube([cube_l, cube_l, cube_h]);
}

// Basic click-on cap for the extruder motor.  Without any holes or decorations.
// Use this as a basis for greating your own faces or other decorations.
module nema17_cap(thickness=0.8, clearance=2, side_l=42.1, corner_l1=6, corner_l2=5.5, corner_h=13.1, clip_l=0.8, wall_h=5) {
  difference() {
    union() {
      // Front plate with wall
      translate([-side_l/2+-thickness, -side_l/2-thickness]) {
        difference() {
          cube([side_l+2*thickness, side_l+2*thickness, thickness+clearance+wall_h]);
          translate([thickness, thickness, thickness]) cube([side_l, side_l, clearance+wall_h+1]);
        }
      }
      for(i = [0:3]) rotate([0, 0, i*90]) {
          translate([side_l/2+thickness, side_l/2+thickness, thickness]) {
            // Corner leg
            linear_extrude(height=corner_h+clearance)
              polygon([[0, 0], [-corner_l1, 0], [0, -corner_l1]]);
            // Corner clip
            translate([0, 0, corner_h+clearance]) {
              linear_extrude(height=thickness)
                polygon([[0, 0], [-corner_l1, 0], [-corner_l1-clip_l, -clip_l], [-clip_l, -corner_l1-clip_l], [0, -corner_l1]]);
            }
            // Corner clearance-block
            translate([-corner_l1, -corner_l1, 0])
              cube([corner_l1, corner_l1, clearance]);
          }
        }
    }

    // Corners to be cut away
    for(i = [0:3]) rotate([0, 0, i*90]) {
        translate([side_l/2+thickness, side_l/2+thickness, -1]) {
            linear_extrude(height=2*thickness+corner_h+clearance+2)
              polygon([[1, 1], [-corner_l2, 1], [1, -corner_l2]]);
        }
      }
  }
}

// Cap for extruder motor with holes for up to 5 gear_front().
module nema17_cap_holed() {
  difference() {
    nema17_cap();
    cylinder(h=10, d=9, center=true);
    for(i = [0:3]) rotate([0, 0, i*90]) {
        translate([17/sqrt(2), 17/sqrt(2), 0])
          cylinder(h=10, d=9, center=true);
      }
  }
}

// Cap for extruder motor with holes for one gear_front(number_of_teeth=14) and two gear_front(number_of_teeth=8).
// Plus Phil Struder's smile! :)
module nema17_cap_face() {
  difference() {
    nema17_cap();

    // Nose
    cylinder(h=10, d=11, center=true);
    // Eyes
    for(i = [0:1]) rotate([0, 0, i*90]) {
        translate([17/sqrt(2), 17/sqrt(2), 0])
          cylinder(h=10, d=9, center=true);
      }
    // Mouth
    translate([0,-3,-1]) {
      intersection() {
        cylinder(d=35, h=1.2);
        union() {
          translate([-25,-9,0]) cube([50, 6, 5]);
          translate([-25,-15.45,0]) cube([50, 6, 5]);
        }
        union() {
          for(i = [-6:5]) translate([0.45/2+i*5.45,-25,0]) cube([5, 25, 5]);
        }
      }
    }
  }
}

// Clip to fasten gears to nema17_cap.
module back_clip(back_d=10.3, middle_d=8.6, back_h=0.8, middle_h=1, cube_h=1.8, cube_l=5) {
  cylinder(d=back_d, h=back_h);
  translate([0,0,back_h]) cylinder(d=middle_d, h=middle_h);
  translate([-cube_l/2, -cube_l/2, back_h+middle_h]) cube([cube_l, cube_l, cube_h]);
}

// Gear to be mounted on the front of nema17_cap.
// Number of teeth on middle gear plus number of teeth on outer gear should be 22 if using holes with distance 17mm like
// the nema17_cap_face().  Experiment with what the sum should be for other distances, if you make your own nema17_cap
// variant.
// You can of course decorate these with eyes, noses, ears, rockets, ducks or whatever you'd like :)
module gear_front(h=2.6, l=5, number_of_teeth=11) {
  difference() {
    gear(number_of_teeth=number_of_teeth,
         circular_pitch=275,
         gear_thickness=h,
         rim_thickness =h,
         hub_thickness =h,
         bore_diameter =0);
    translate([-l/2,-l/2,-1]) cube([l, l, h+2]);
  }
}

// A gear with an eye on top.  For nema17_cap_face().
module gear_front_eye() {
  color("blue") gear_front(h=2.8, number_of_teeth=8);
  color("white") translate([0,0,2.4]) cylinder(h=1, d=8.5);
  color("black") translate([1.9,1.9,3.5]) cylinder(h=0.5, d=3);
}
// Two gear_front_eye() with their matching "nose" gear.
module gear_front_eyes() {
  // Nose
  color("blue") gear_front(number_of_teeth=14);
  // Eyes
  for(i = [0:1]) rotate([0,0,i*90]) translate([0,17,0]) rotate([0,0,i*180/8]) gear_front_eye();
}

// The complete Phil Struder! (Without the 3 x back_clip()) Not to be printed in one piece of course.
module complete_face() {
  gear_front_eyes();
  rotate([180,0,-135]) nema17_cap_face();
}
